/*
 * radiomodul.cpp
 *  Author: roland
 * Stations in EEPROM: 92.5 as fixcomma 925
 * EEPROM:			0			0		1			2				3				4  ...
 *            activeSender		free   Station1MSB	Station1LSB		Station2MSB		Station2LSB
 */
#include "radiomodul.h"
#include <EEPROM.h>
#include "lib/TEA5767.h"
#include "traceit.h"
#include "printf.h"
#include <Arduino.h>

int Radiomodul::hilo_optimal (unsigned long freq) {
    int signal_high = 0;
    int signal_low = 0;
    unsigned char buf[5];
    TEA5767::set_frequency (1, (double) (freq + 450000) / 1000000);
    DELAY (30);
    // Read the signal level
    if (read_status (buf) == 1) {
        signal_high = signal_level (buf);
    }
    TEA5767::set_frequency (0, (double) (freq - 450000) / 1000000);
    DELAY (30);
    if (read_status (buf) == 1) {
        signal_low = signal_level (buf);
    }
    return (signal_high < signal_low) ? 1 : 0;
}
void Radiomodul::saveAndSet(){
    EEPROM.write (0, activeSenderPlace);
    byte fLSB = EEPROM.read (activeSenderPlace*2);
    byte fMSB = EEPROM.read (activeSenderPlace*2+1);
	uint16_t f = fMSB*256+fLSB; 
    if (f != 0) set_frequency (f);
    TRACE ("Station:")
    TRACE (activeSenderPlace);
}
void  Radiomodul::prevStation () {
	if (activeSenderPlace==0) return; 
    activeSenderPlace = activeSenderPlace == 1 ? NR_OF_STATIONS : activeSenderPlace - 1;
    saveAndSet(); 
}

void  Radiomodul::nextStation () {
	if (activeSenderPlace==0) return;
	activeSenderPlace = activeSenderPlace == NR_OF_STATIONS ? 0 : activeSenderPlace + 1;
	saveAndSet();
}


void  Radiomodul::decFrequency (unsigned char *buf) {
    double rounded_freq;
    double freq_available = frequency_available (buf);
    rounded_freq = floor (freq_available / 100000 - 1.5) / 10;
    if (rounded_freq <= BANDLIMIT_LOW) rounded_freq = BANDLIMIT_HIGH;
    set_frequency (rounded_freq);
    TRACE ("frequ:")
    TRACE (freq_available);
    TRACE ("frequ-neu:")
    TRACE (rounded_freq);
    }



void  Radiomodul::incFrequency (unsigned char *buf) {
    double rounded_freq;
    double freq_available = frequency_available (buf);
    rounded_freq = floor (freq_available / 100000 + 1.5) / 10;
    if (rounded_freq >= BANDLIMIT_HIGH) rounded_freq = BANDLIMIT_LOW;
    set_frequency (rounded_freq);
    TRACE ("frequ:")
    TRACE (freq_available);
    TRACE ("frequ-neu:")
    TRACE (rounded_freq);
}

void Radiomodul::clearAllStations() {
    TRACE ("EEPROM CLEAR")
	activeSenderPlace=0; 
	activeSenderPlaceFreuqency=92;
    for (int i = 0; i < (NR_OF_STATIONS+1)*2; i++)
        EEPROM.write (i, 0);
}

void Radiomodul::setActiveStation (int stationNr) {
		
    if (stationNr == 0) //no valid Sender
        Radio.set_frequency (92.0); //OE1
    else {
        Radio.activeSenderPlace = stationNr;
		double f = (EEPROM.read (stationNr*2)*256+EEPROM.read (stationNr*2+1))/10; 
        Radio.activeSenderPlaceFreuqency = f;
        Radio.set_frequency (Radio.activeSenderPlaceFreuqency);
    }
}
void Radiomodul::set_frequency (uint16_t f) {
	double freq = f/10.0;  // fixcomma => float
    HILO = hilo_optimal ( (unsigned long) (freq * 1000000) );
    TEA5767::set_frequency (HILO, freq);
}
void Radiomodul::saveStationToEEPROM(){
	double current_freq; 
	//Todo: short feedback to user e.g. 2s Mute
	if (read_status(Radio.buf) == 1) {
		 current_freq = floor(Radio.frequency_available(buf) / 100000 + .5) / 10;
#ifdef ENABLE_TWI2SERIAL
		 current_freq = 93.5d;  
#endif
	     Radio.activeSenderPlace++;
		 Radio.activeSenderPlaceFreuqency = current_freq;
		 EEPROM.write(0,Radio.activeSenderPlace);
		 int16_t  cf = (int16_t)(current_freq*10); 
		 int8_t fHIGH = cf/256;
		 int8_t fLOW  = cf%256;
		 
		 EEPROM.write(activeSenderPlace*2,fHIGH);
		 EEPROM.write(activeSenderPlace*2+1,fLOW);
		 set_frequency (Radio.activeSenderPlaceFreuqency);
	}
	
	 
}
void Radiomodul::searchNextStation() {
	TRACE ("searchNextstation")
	search_down(buf);
	while (ready(buf) == 0)
	{
		//wait for Auto Search ready
		DELAY (500)
		while (read_status(buf) == 0);
		if (timeout==true) return;  //don't hang up here
	}
}


void Radiomodul::searchPrevStation() {
	TRACE("searchPrevStation")
	search_up(buf);
	while (ready(buf) == 0) {
		DELAY(100)
		while (read_status(buf) == 0);
		if (timeout==true) return;  //don't hang up here
	}
}

Radiomodul Radio;
